#include <iostream>
#include <iomanip>
#include <fstream>
#include <string>
#include <cstring>
#include <cstdlib>
#include <signal.h>
#include <time.h>
#include <sys/stat.h>

extern "C" {
#include "../vxi11/vxi11_user.h"
}

#include "TFile.h"
#include "TTree.h"

#include "tek_scope.h"
#define BUF_LEN 100000

bool isKill=false;
void handler(int signum)
{
  isKill=true;
}

int main(int argc, char **argv)
{
  //IP #ofWfm ofname
  signal(SIGINT, handler);
  
  struct stat st;
  int ret = 0;

  static char	*device_ip;
  static char	*device_name;

  char fname[1024];//output file name
  int nwfm=0;//# of waveform to capture. 0 or negative:infintie loop
  int chs;//# channel assignement with 2-bit encoding. For example, 2=0b10:ch2, 3=0b11:ch2 and ch1
  TFile* tf = 0;
  
  if (! (argc == 5 || argc==6 ) ) {
    std::cerr << "usage: "
	      << argv[0]
	      << " ip_addr"
	      << " #_of_wfm(<=0 : infintie loop)"
	      << " channel_assign(2-bit enconding) "
	      << " output_file_name"
	      << "[ timeout(in ms) ]"
	      << std::endl;
    std::cerr << "Example of 2-bit encoding : "
	      << "2=0b10:ch2, 3=0b11:ch2 and ch1" << std::endl;
    return 1;
  }

  device_ip = argv[1];
  
  
  nwfm=std::atoi(argv[2]);
  chs=std::atoi(argv[3]);
  if( ! (chs>=1 && chs<=15) ) {
    std::cerr << "Channel assignment error : " << chs << std::endl;
    std::cerr << "Example of 2-bit encoding : "
	      << "2=0b10:ch2, 3=0b11:ch2 and ch1" << std::endl;
    return 1;
  }
  std::sprintf(fname,argv[4]);
  ret=stat(fname,&st);
  if(ret==0) {
    std::cerr << "File exists. " << fname << std::endl;
    return 1;
  } else {
    tf = new TFile(fname,"RECREATE");
  }
  
  unsigned long	timeout=10000; /* in ms (= 10 seconds) */
  if(argc==5) {
    std::cerr << "Defualt timeout for waveform capture is " << timeout << " ms." << std::endl;
  } else {
    timeout=std::atoi(argv[5]);
  }
  
  
  VXI11_CLINK *clink;
  ret=0;
  ret=vxi11_open_device(&clink, device_ip, device_name);
  
  if (ret != 0) {
    std::cerr << "Error: could not open device "
	      << device_ip
	      << " , quitting" << std::endl;
    return 2;
  }
  
  int isTDS3000=tek_scope_is_TDS3000(clink);
  if(isTDS3000) {
    std::cerr << "Hello,this is TDS3000 oscilosocpe." << std::endl;
  } else {
    std::cerr << "Hello ,this is probably MSOXXXX oscilosocpe." << std::endl;
  }
  
  
  char 		wfname[1024];
  char 		wfiname[1024];
  const char* cch[4]={"CH1","CH2","CH3","CH4"};
  int active[4]={1,1,1,1};
  active[3]=chs/8;
  chs=chs-active[3]*8;
  active[2]=chs/4;
  chs=chs-active[2]*4;
  active[1]=chs/2;
  chs=chs-active[1]*2;
  active[0]=chs;
  int npoints=10000;
  int clear_sweeps=1;
  char		channel[20];
  
  if (tek_scope_init(clink) !=0 ) {
    std::cerr << "GETWFM failed." << std::endl;
    return false;
  }
  
  char	cmd[1024];
  std::sprintf(cmd,"HOR:RECORDLENGTH %d",npoints);
  vxi11_send(clink,cmd);
  long actual_npoints =vxi11_obtain_long_value(clink, "HOR:RECORDLENGTH?");

  if(actual_npoints<npoints) {
    npoints=actual_npoints;
  }
  
  vxi11_send(clink, "ACQUIRE:MODE SAMPLE");
  long buf_size = tek_scope_set_for_capture(clink, clear_sweeps, timeout);
  char* buf=new char[buf_size];
  npoints = buf_size/2;

  long no_of_bytes[4];
  double yoff[4];
  double yzero[4];
  double vgain[4];
  double voffset[4];
  double hinterval[4];
  long data_start[4];
  double xzero[4];
  double hoffset[4];
  


  bool isInfLoop=false;
  if(nwfm<=0) {
    isInfLoop=true;
    nwfm=1;
  }
  double* x = new double[npoints];
  double* y[4];
  for(int j=0;j<4;j++) {
    y[j]= new double[npoints]; 
  }
  TTree* tr = new TTree("tr","");
  char bn[1024];
  char bs[1024];
  int ut;
  tr->Branch("ut",&ut,"ut/I");
  std::sprintf(bs,"x[%d]/D",npoints);
  tr->Branch("x",x,bs);
  for(int j=0;j<4;j++) {
    if(active[j]) {
      std::sprintf(bn,"y%d",j);
      std::sprintf(bs,"y%d[%d]/D",j,npoints);
      tr->Branch(bn,y[j],bs);
    }
  }


  time_t curT=0;
  struct tm* curtm=0;
  
  short* val=0;
  int cnt=0;
  int div=int(nwfm/100.);
  for(int i=0;i<nwfm;i++) {
    curT = time(0);
    ut = curT;
    
    
    if(!isInfLoop) {
      if(i%div==0) {
	curtm=localtime(&curT);
	std::cout << std::setw(3) << i/div << " % ";
	std::cout << std::setfill('0') << std::setw(4) << curtm->tm_year+1900 << "/";
	std::cout << std::setfill('0') << std::setw(2) << curtm->tm_mon+1 << "/";
	std::cout << std::setfill('0') << std::setw(2) << curtm->tm_mday << " ";
	std::cout << std::setfill('0') << std::setw(2) << curtm->tm_hour << ":";
	std::cout << std::setfill('0') << std::setw(2) << curtm->tm_min << ":";
	std::cout << std::setfill('0') << std::setw(2) << curtm->tm_sec << std::endl;
	std::cout << std::setfill(' ');
      }
    } else {
      i--;
    }
    
    

    if(isKill) break;
    
    vxi11_send(clink, "ACQUIRE:STATE 1");
    long opc_value = vxi11_obtain_long_value_timeout(clink, "*OPC?", timeout);
    if (opc_value != 1) {
      std::cerr << "OPC? request returned " << opc_value <<
	" (should be 1), maybe you need a longer timeout?" << std::endl;
      std::cerr << "Not grabbing any data, returning -1" << std::endl;
      return -1;
    }
    bool isFirst[4]={true,true,true,true};
    for(int j=0;j<4;j++) {
      if(active[j]) {
	int ret=0;
	std::sprintf(cmd,"DATA:SOURCE %s",const_cast<char*>(cch[j]));
	ret=vxi11_send(clink, cmd);
        if(ret < 0) {
	  std::cerr << "error, could not send DATA SOURCE cmd, quitting..." << std::endl;
	  return ret;
	}
	vxi11_send(clink, "CURVE?");
        long bytes_returned=vxi11_receive_data_block(clink, buf, buf_size, timeout);
	/////////////////////////////
	
	if (bytes_returned <=0) {
	  std::cerr << "Problem reading the ch "
		    << j << ", "
		    << cnt << " th data, skipping..." << std::endl;
	  continue;
	}

	if(isFirst[j]) {
	  no_of_bytes[j] = tek_scope_calculate_no_of_bytes(clink,isTDS3000, timeout);
	  yoff[j] = vxi11_obtain_double_value(clink, "WFMPRE:YOFF?");
	  yzero[j] = vxi11_obtain_double_value(clink, "WFMPRE:YZERO?");
	  vgain[j] = vxi11_obtain_double_value(clink, "WFMPRE:YMULT?");
	  voffset[j] = -(yoff[j] * vgain[j]) + yzero[j];
	  hinterval[j] = vxi11_obtain_double_value(clink, "WFMPRE:XINCR?");
	  data_start[j]= vxi11_obtain_long_value(clink, "DATA:START?");
	  xzero[j] = vxi11_obtain_double_value(clink, "WFMPRE:XZERO?");
	  hoffset[j] = xzero[j] + (((double) (data_start[j] - 1)) * hinterval[j]);
	  isFirst[j]=false;
	}
	for(int k=0;k<npoints;k++){
	  x[k] = k*hinterval[j]+hoffset[j];
	}
      }
	
      //binary conversion
      for(int k=0;k<npoints;k++){
	val =(short*)(buf + 2*k);
	y[j][k] = (*val)*vgain[j]+voffset[j];
      }
    }
    
    // for(int k=0;k<npoints;k++){
    //   std::cout << x[k] << "\t";
    //   for(int j=0;j<4;j++) {
    // 	if(active[j]) {
    // 	  std::cout << y[j][k] << "\t";
    // 	}
    //   }
    //   std::cout << std::endl;
    // }

    tr->Fill();
    cnt++;
  }
  tr->Write();
  tf->Close();
  delete[] buf;
  delete x;
  for(int j=0;j<4;j++) {
    delete y[j];
  }
  ret=vxi11_close_device(clink,device_ip);

  
  return 0;
}

